/************************************************************
 *  Martin Toepfer     ***     PROJECT : Lingvisto          *
 *  Wissmannstr. 5     ***                                  *
 *  D - 22041 Hamburg  ***     File    : datastructure.h    *
 *  Germany            ***     Date    : 04.04.1997         *
 ************************************************************/
#ifndef DATASTRUCTURE
#define DATASTRUCTURE

#define CHANGED 1              /* indicates changes in file */
#define UNCHANGED 0         /* indicates no changes in file */
#define FOREIGN 2                   /* foreign regular test */
#define NATIVE 4                     /* native regular test */
#define MFOREIGN 1          /* foreign multiple choice test */ 
#define MNATIVE 3            /* native multiple choice test */
#define MAXLINE 250         /* maximum wordlength, this has */
                             /* to fit in 'unsigned char'   */

/**********************************************************************
 *            basic wordlist and filehead definitions                 *
 **********************************************************************/
/************************************************************
 * struct words : one pair of words                         *
 *                                                          *
 * native  : pointer to native language word                *
 * foreign : pointer to foreign language word               *
 ************************************************************/
typedef struct words WORDS;
struct words {
  char *native;
  char *foreign;
};   

/************************************************************
 * struct wordnode : one listnode of one pair of words      *
 *                                                          *
 * pair   : pair of words, type 'struct words'              *
 * next   : pointer to next 'struct wordnode'               *
 * prev   : pointer to previous 'struct wordnode'           *
 ************************************************************/
typedef struct wordnode *NODE;
struct wordnode {
  WORDS pair;
  NODE next;
  NODE prev;
}; 

/************************************************************
 * struct lvo_file : head of each file of words             *
 *                                                          *
 * filename     : pointer to filename                       *
 * filename_bak : pointer to filename backup                *
 * state        : indicator for unsaved changes             *
 * wordlist     : pointer to wordlist of 'struct wordnode'  *
 ************************************************************/
struct lvo_file {
  char *filename;
  char *filename_bak;
  unsigned char state;
  NODE wordlist; 
};

/**********************************************************************
 *              function prototypes (some as macros)                  *
 **********************************************************************/
#define set_state(s) head.state = ((unsigned char) s)
#define get_state() head.state
void set_filename(char *filename);
#define get_filename() (head.filename != NULL ? head.filename : "unnamed file")
void restore_filename(void);
#define set_wordlist(w) head.wordlist = ((NODE) w)
#define get_wordlist() head.wordlist
void set_words(char *native, char *foreign, NODE node);
void init_ds(void);
NODE n_alloc(void);
void n_free(NODE node);

#endif
